@@ -1,17 +1,19 @@
 from collections import namedtuple
+from typing import List, NamedTuple, Optional, Tuple, Union
 
 from pybliometrics.scopus.superclasses import Retrieval
-from pybliometrics.scopus.utils import get_link
+from pybliometrics.scopus.utils import chained_get, check_parameter_value,\
+    get_link, make_int_if_possible
 
 
 class SerialTitle(Retrieval):
     @property
-    def aggregation_type(self):
+    def aggregation_type(self) -> str:
         """The type of the source."""
         return self._entry['prism:aggregationType']
 
     @property
-    def citescoreyearinfolist(self):
+    def citescoreyearinfolist(self) -> Optional[List[Tuple[int, float]]]:
         """A list of two tuples of the form (year, cite-score).  The first
         tuple represents the current cite-score, the second tuple
         represents the tracker cite-score."""
@@ -19,29 +21,31 @@ class SerialTitle(Retrieval):
             d = self._entry['citeScoreYearInfoList']
         except KeyError:
             return None
-        current = (d['citeScoreCurrentMetricYear'], d['citeScoreCurrentMetric'])
-        tracker = (d['citeScoreTrackerYear'], d['citeScoreTracker'])
+        current = (int(d['citeScoreCurrentMetricYear']),
+                   float(d['citeScoreCurrentMetric']))
+        tracker = (int(d['citeScoreTrackerYear']),
+                   float(d['citeScoreTracker']))
         return [current, tracker]
 
     @property
-    def eissn(self):
+    def eissn(self) -> Optional[str]:
         """The electronic ISSN of the source."""
         return self._entry.get('prism:eIssn')
 
     @property
-    def issn(self):
+    def issn(self) -> Optional[str]:
         """The ISSN of the source."""
         return self._entry.get('prism:issn')
 
     @property
-    def oaallowsauthorpaid(self):
+    def oaallowsauthorpaid(self) -> Optional[str]:
         """Whether under the Open-Access policy authors are allowed to pay."""
         return self._entry.get('oaAllowsAuthorPaid')
 
     @property
-    def openaccess(self):
+    def openaccess(self) -> Optional[int]:
         """Open Access status (0 or 1)."""
-        return self._entry.get('openaccess')
+        return make_int_if_possible(chained_get(self._entry, ['openaccess']))
 
     @property
     def openaccessstartdate(self):
@@ -49,52 +53,52 @@ class SerialTitle(Retrieval):
         return self._entry.get('openaccessStartDate')
 
     @property
-    def openaccesstype(self):
+    def openaccesstype(self) -> Optional[str]:
         """Open Archive status (full or partial)."""
         return self._entry.get('openaccessType')
 
     @property
-    def openaccessarticle(self):
-        """Open Access status (boolean)."""
+    def openaccessarticle(self) -> Optional[bool]:
+        """Open Access status."""
         return self._entry.get('openaccessArticle')
 
     @property
-    def openarchivearticle(self):
-        """Open Archive status (boolean)."""
+    def openarchivearticle(self) -> Optional[bool]:
+        """Open Archive status."""
         return self._entry.get('openArchiveArticle')
 
     @property
-    def openaccesssponsorname(self):
+    def openaccesssponsorname(self) -> Optional[str]:
         """The name of the Open Access sponsor."""
         return self._entry.get('openaccessSponsorName')
 
     @property
-    def openaccesssponsortype(self):
+    def openaccesssponsortype(self) -> Optional[str]:
         """The type of the Open Access sponsor."""
         return self._entry.get('openaccessSponsorType')
 
     @property
-    def openaccessuserlicense(self):
+    def openaccessuserlicense(self) -> Optional[str]:
         """The User license."""
         return self._entry.get('openaccessUserLicense')
 
     @property
-    def publisher(self):
+    def publisher(self) -> str:
         """The publisher of the source."""
         return self._entry['dc:publisher']
 
     @property
-    def scopus_source_link(self):
+    def scopus_source_link(self) -> str:
         """URL to info site on scopus.com."""
         return get_link(self._entry, 0, ["link"])
 
     @property
-    def self_link(self):
+    def self_link(self) -> str:
         """URL to the source's API page."""
         return get_link(self._json, 0, ["link"])
 
     @property
-    def sjrlist(self):
+    def sjrlist(self) -> Optional[List[Tuple[int, float]]]:
         """The SCImago Journal Rank (SJR) indicator as list of
         (year, indicator)-tuples.  See
         https://www.scimagojr.com/journalrank.php.
@@ -102,7 +106,7 @@ class SerialTitle(Retrieval):
         return _parse_list(self._entry, "SJR")
 
     @property
-    def sniplist(self):
+    def sniplist(self) -> Optional[List[Tuple[int, float]]]:
         """The Source-Normalized Impact per Paper (SNIP) as list of
         (year, indicator)-tuples.  See
         https://blog.scopus.com/posts/journal-metrics-in-scopus-source-normalized-impact-per-paper-snip.
@@ -110,73 +114,74 @@ class SerialTitle(Retrieval):
         return _parse_list(self._entry, "SNIP")
 
     @property
-    def source_id(self):
+    def source_id(self) -> int:
         """The Scopus ID of the source."""
-        return self._entry['source-id']
+        return int(self._entry['source-id'])
 
     @property
-    def subject_area(self):
+    def subject_area(self) -> Optional[List[NamedTuple]]:
         """List of named tuples of subject areas in the form
         (area, abbreviation, code) of the source.
         """
         area = namedtuple('Subjectarea', 'area abbreviation code')
-        areas = [area(area=item['$'], code=item['@code'],
+        areas = [area(area=item['$'], code=int(item['@code']),
                       abbreviation=item['@abbrev'])
                  for item in self._entry["subject-area"]]
         return areas or None
 
     @property
-    def title(self):
+    def title(self) -> str:
         """The title of the source."""
         return self._entry['dc:title']
 
-    def __init__(self, issn, refresh=False, view="ENHANCED", years=None):
+    def __init__(self,
+                 issn: Union[int, str],
+                 refresh: Union[bool, int] = False,
+                 view: str = "ENHANCED",
+                 years: str = None
+                 ) -> None:
         """Interaction with the Serial Title API.
 
-        Parameters
-        ----------
-        issn : str or int
-            The ISSN or the E-ISSN of the source.
+        :param issn: The ISSN or the E-ISSN of the source.
+        :param refresh: Whether to refresh the cached file if it exists or not.
+                        If int is passed, cached file will be refreshed if the
+                        number of days since last modification exceeds that value.
+        :param view: The view of the file that should be downloaded.  Allowed
+                     values: BASIC, STANDARD, ENHANCED.  For details see
+                     https://dev.elsevier.com/sc_serial_title_views.html.
+        :param years: A string specifying a year or range of years (combining
+                      two years with a hyphen) for which yearly metric data
+                      (SJR, SNIP, yearly-data) should be looked up for.  If
+                      None, only the most recent metric data values are
+                      provided. Note: If not None, refresh will always be True.
 
-        refresh : bool or int (optional, default=False)
-            Whether to refresh the cached file if it exists or not.  If int
-            is passed, cached file will be refreshed if the number of days
-            since last modification exceeds that value.
-
-        view : str (optional, default="ENHANCED")
-            The view of the file that should be downloaded.  Allowed values:
-            BASIC, STANDARD, ENHANCED.  For details see
-            https://dev.elsevier.com/guides/SerialTitleViews.htm.
-
-        years : str (optional, default=None)
-            A string specifying a year or range of years (combining two
-            years with a hyphen) for which yearly metric data (SJR, SNIP,
-            yearly-data) should be looked up for.  If None, only the
-            most recent metric data values are provided.
-            Note: If not None, refresh will always be True.
-
-        Examples
-        --------
-        See https://pybliometrics.readthedocs.io/en/stable/examples/SerialTitle.html.
+        Raises
+        ------
+        ValueError
+            If any of the parameters `refresh` or `view` is not
+            one of the allowed values.
 
         Notes
         -----
         The directory for cached results is `{path}/{view}/{source_id}`,
-        where `path` is specified in `~/.scopus/config.ini`.
+        where `path` is specified in your configuration file.
         """
         # Checks
-        allowed_views = ('BASIC', 'STANDARD', 'ENHANCED')
-        if view not in allowed_views:
-            raise ValueError('view parameter must be one of ' +
-                             ', '.join(allowed_views))
-        # Load json
-        self._id = str(issn)
-        self._years = years
+        check_parameter_value(view, ('BASIC', 'STANDARD', 'ENHANCED'), "view")
+        self._view = view
+
         # Force refresh when years is specified
         if years:
             refresh = True
-        Retrieval.__init__(self, identifier=self._id, view=view, date=years,
-                           api='SerialTitle', refresh=refresh)
+        self._refresh = refresh
+
+        # Load json
+        self._id = str(issn)
+        self._years = years
+        Retrieval.__init__(self, identifier=self._id, date=years,
+                           api='SerialTitle')
+
+        # Parse json
         self._json = self._json['serial-metadata-response']
         self._entry = self._json['entry'][0]
 
@@ -209,7 +214,8 @@ class SerialTitle(Retrieval):
 def _parse_list(d, metric):
     """Auxiliary function to parse SNIP and SJR lists."""
     try:
-        values = [(r['@year'], r['$']) for r in d[metric + "List"][metric]]
+        values = [(int(r['@year']), float(r['$'])) for r
+                  in d[metric + "List"][metric]]
         return sorted(set(values))
     except (KeyError, TypeError):
         return None
