@@ -8,7 +8,7 @@ from nose.tools import assert_equal, assert_true
 from pybliometrics.scopus import AuthorSearch
 
 s1 = AuthorSearch('authlast(selten) and authfirst(reinhard)', refresh=30)
-s2 = AuthorSearch('authlast(selten)', download=False)
+s2 = AuthorSearch('authlast(selten)', download=False, refresh=True)
 
 
 def test_authors():
@@ -19,7 +19,7 @@ def test_authors():
     Author = namedtuple('Author', order)
     expected = Author(eid='9-s2.0-6602907525', surname='Selten',
         initials='R.', givenname='Reinhard', affiliation='Universität Bonn',
-        documents='74', affiliation_id='60007493', city='Bonn',
+        documents=74, affiliation_id='60007493', city='Bonn',
         country='Germany', areas='ECON (73); MATH (19); BUSI (16)')
     assert_equal(s1.authors[0], expected)
 
